/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Netscape security libraries.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are 
 * Copyright (C) 2003 Netscape Communications Corporation.  All
 * Rights Reserved.
 * 
 * Contributor(s):
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 *
 * shsign.h - header to build softoken signature files.
 *
 * $Id: shsign.h,v 1.1 2003/01/28 18:50:02 relyea%netscape.com Exp $
 */

#ifndef _SHSIGN_H_
#define _SHSIGN_H_

#define SGN_SUFFIX ".chk"
#define  NSS_SIGN_CHK_MAGIC1 0xf1
#define  NSS_SIGN_CHK_MAGIC2 0xc5
#define  NSS_SIGN_CHK_MAJOR_VERSION 0x01
#define  NSS_SIGN_CHK_MINOR_VERSION 0x02

#endif /* _SHSIGN_H_ */
